/*----------------------------------------------------------------------------*/
/*-------                         INFORMATION                         --------*/
/*----------------------------------------------------------------------------*/
/*----------------------------------------------------------------------------*/

// Cable nailing clip factory
// Written by Kakiemon @ Thingiverse
// Rolf Jethon
// Licensed under the GPL v3 Terms and conditions.

// HOW TO USE THIS FILE :

// -- Solution 1 :
// Open this file with OpenScad software (free and open-source)
// http://www.openscad.org/downloads.html
// Choose your parameters in the SETTINGS chapter (see below)
// Press F5 to compile and see the resulting clip
// Check your hook's global size in the console window (below viewing area)
// If OK, press F6 to render
// Go to : File => Export => Export as STL : choose filename and location
// Save the STL file of your customized clip
// Send this STL file to your 3D printer

// -- Solution 2 :
// This file had been optimized for Thingiverse Customizer
// You will need to have a Thingiverse Account, then :
// Go to object's page on Thingiverse
// Click on the button "Open in Customizer"
// Choose your settings in parameters tabs
// Save the STL file of your customized clip
// Process and print this STL file for your 3D printer

/*----------------------------------------------------------------------------*/
/*-------                           SETTINGS                          --------*/
/*----------------------------------------------------------------------------*/

/* [Sizes in mm] */
//Diameter of the cable to hold - Range 0.1 to 100
cablediameter=7;//[0.1:0.1:100]
//Diameter of the nail hole - Range 0 to 30
naildiameter=2.2;//[0:0.1:30] 
//Thickness on the clamp wall - Range 0.1 to 100
wall=1.5;
//Width of the clamp - Range 0.1 to 100
width=6;//[0.1:0.1:100]
//Thickness of the wall around the nail hole - Range 0.1 to 30
nailwall=2;//[0:0.1:30]
//Save print material if 1
savematerial=true;
// Depht of the sunk hole - Range 0 to 100
sunkdepht=.5;
//Diameter of the sunk hole - Range 0 to 100
sunkdiameter=4.1;
//Angle of the sunk hole
sunkangle=30;//[89] 
//Add print support in % of the nail bushing diameter
printsupportpercent=0;//[100] 
//Cut off from the bottom - usually zero - Range 0 to 50
lowercutoff=0;//[0:0.1:50]
//Cut off from the top - usually zero - Range 0 to 80
uppercutoff=0;//[0:0.1:80]
//Number of cables to hold
numcables=2;//[1:10]
//Number of Nails
numnails=2;//[1:2]
$fn=100; 

/*------------------------------------------------------------------------------

NOTES about how to use :
You may change any of the above parameters using the parameter range below
cablediameter       - the diameter of the cable to hold > 0
naildiameter        - the diameter of the nail hole 0 - sensful
width               - the width of the clamp 
wall                - the thickness of the cable holding bow > 0
nailwall            - the thickness of the wall around the nail 0 - sensful
savematerial        - if true, less material is needed to print false or true
sunkdepht           - the depht of the nail head sunk hole 0 - clamp height
sunkangle           - the angle of the sunken edge inside the nail hole 0 - 89
printsupportpercent - mainly for savematerial true - adds a support in the width in 
                      percent of the nail bushing diameter 0 - 100
lowercutoff         - how much to cut off from the bottom of the clamp to apply
                      some pressure onto the cable
uppercutoff         - how much to cut off from the top of the clamp
numcables           - how many cabes to hold with one clamp
numnails            - how many nails to fix the clamp

Console will display warning messages if some values are illogic.

------------------------------------------------------------------------------*/

//----------------------------------------------------------------------------
//-------                             CODE                            --------
//----------------------------------------------------------------------------

//____________________________________________________________________________

if (naildiameter>sunkdiameter){
    echo (" WARNING : naildiameter must be smaller or equal to sunkdiameter ");
}    
sunkdistance=(sunkdiameter-naildiameter)/2;
sunkdeph=(sunkdistance * tan(sunkangle));
innersunkdiameter=naildiameter;
outerdiameter=cablediameter+wall*2;
nailnosediameter=naildiameter+nailwall*2;
num_cables=numcables-1;
difference(){
  if (savematerial){  
    bodywithoutholes();
  } else {
    hull(){
      bodywithoutholes();
    }
  }
  cableholes();  
  startnailhole();
  if (numnails==2){
    translate([num_cables*(cablediameter+wall),0,0])    
    endnailhole();  
  }  
  //cutoff lower side
  rotate([90,0,0])
  translate([-(outerdiameter/2+nailnosediameter),-1-nailnosediameter/2,cablediameter/2-lowercutoff+0.01])
  cube([numcables*outerdiameter+nailnosediameter*2+1,nailnosediameter+width+2,lowercutoff+1]); 
  //cutoff upper side  
  rotate([90,0,0])
  translate([-(outerdiameter/2+nailnosediameter),-1-nailnosediameter/2,-outerdiameter/2-1.01])
  cube([numcables*outerdiameter+nailnosediameter*2+1,nailnosediameter+width+2,uppercutoff+1]);  
}
module bodywithoutholes(){ 
  for(a=[0:cablediameter+wall:(num_cables)*(cablediameter+wall)]){
    translate([a,0,0])
    bodywithouthole(); 
    startbushing();
    if (numnails==2){
      translate([(num_cables)*(cablediameter+wall),0,0])
      endbushing();  
    }
  }
}
module cableholes(){
  for(a=[0:cablediameter+wall:num_cables*(cablediameter+wall)]){
    translate([a,0,0])
    cablehole();    
  }
}

module cablehole() {
  union(){   // inner border of clamp
    translate([0,0,-50])  
    cylinder (d=cablediameter,h=width+100);
    translate([-cablediameter/2,-outerdiameter/2,-50])
    cube([cablediameter,outerdiameter/2,width+100]);
  }    
}  
module bodywithouthole() {
    union(){
      difference(){ // outer border of clamp
        union(){
          difference() {
            cylinder (d=outerdiameter,h=width);
            translate([-outerdiameter/2-.5,-outerdiameter/2-1,-.5])
            cube([outerdiameter+1,outerdiameter/2+1,width+1]);
          }    
          translate([-outerdiameter/2,-cablediameter/2,0])
          cube([outerdiameter,cablediameter/2,width]);
        } 
      }  
    }    
}    

module startbushing(){
  // Nailbushing
  hull() {
  translate([-(nailnosediameter/2+cablediameter/2),outerdiameter/2,width/2])
  rotate([90,0,0])
  cylinder(d=nailnosediameter,h=outerdiameter/2+cablediameter/2);
  // Printsupport  
  translate([-(nailnosediameter/2+cablediameter/2+nailnosediameter/100*printsupportpercent/2),outerdiameter/2,0])
  rotate([90,0,0])
  cube([nailnosediameter/100*printsupportpercent,width,outerdiameter/2+cablediameter/2]);   
  }
}
module endbushing(){
  // Nailbushing
  hull () {
    translate([(nailnosediameter/2+cablediameter/2),outerdiameter/2,width/2])
    rotate([90,0,0])
    cylinder(d=nailnosediameter,h=outerdiameter/2+cablediameter/2);
    // Printsupport  
    translate([(nailnosediameter/2+cablediameter/2-nailnosediameter/100*printsupportpercent/2),outerdiameter/2,0])
    rotate([90,0,0])
   cube([nailnosediameter/100*printsupportpercent,width,outerdiameter/2+cablediameter/2]);   
  }
}  
module startnailhole() {
  //Nailhole
  translate([-(nailnosediameter/2+cablediameter/2),outerdiameter/2+.5,width/2])
  rotate([90,0,0])
  cylinder(d=naildiameter,h=outerdiameter/2+cablediameter/2+1);
  //Nailsunk - conical part
  translate([-(nailnosediameter/2+cablediameter/2),outerdiameter/2-sunkdepht+.02-uppercutoff,width/2])
  rotate([90,0,0])
  cylinder(h=sunkdeph,d1=sunkdiameter,d2=innersunkdiameter);
  //Nailsunk - cylindrical part
  translate([-(nailnosediameter/2+cablediameter/2),outerdiameter/2+.02-uppercutoff,width/2])
  rotate([90,0,0])
  cylinder(h=sunkdepht+.02,d=sunkdiameter);
}
module endnailhole() {
  //Nailhole
  translate([(nailnosediameter/2+cablediameter/2),outerdiameter/2+.5,width/2])
  rotate([90,0,0])
  cylinder(d=naildiameter,h=outerdiameter/2+cablediameter/2+1);
  //Nailsunk - conical part
  translate([(nailnosediameter/2+cablediameter/2),outerdiameter/2-sunkdepht+.02-uppercutoff,width/2])
  rotate([90,0,0])
  cylinder(h=sunkdeph,d1=sunkdiameter,d2=innersunkdiameter);
  //Nailsunk - cylindrical part
  translate([(nailnosediameter/2+cablediameter/2),outerdiameter/2+.02-uppercutoff,width/2])
  rotate([90,0,0])
  cylinder(h=sunkdepht+0.02,d=sunkdiameter);
}    
